﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.FitnessLandscape.BoxCharts {
  
  [StorableClass]
  public class EmptyBoxChartElementGenerator : ParameterizedNamedItem, IBoxChartElementGenerator {

    public ValueParameter<IntValue> XParameter {
      get { return (ValueParameter<IntValue>)Parameters["X"]; }
    }
    public ValueParameter<IntValue> YParameter {
      get { return (ValueParameter<IntValue>)Parameters["Y"]; }
    }
    public ValueParameter<IntValue> RowSpanParameter {
      get { return (ValueParameter<IntValue>)Parameters["RowSpan"]; }
    }
    public ValueParameter<IntValue> ColSpanParameter {
      get { return (ValueParameter<IntValue>)Parameters["ColSpan"]; }
    }

    public int X { get { return XParameter.Value.Value; } }
    public int Y { get { return YParameter.Value.Value; } }
    public int ColSpan { get { return ColSpanParameter.Value.Value; } }
    public int RowSpan { get { return RowSpanParameter.Value.Value; } }

    #region Construction & Cloning
    [StorableConstructor]
    protected EmptyBoxChartElementGenerator(bool deserializing) : base(deserializing) { }
    protected EmptyBoxChartElementGenerator(EmptyBoxChartElementGenerator original, Cloner cloner) : base(original, cloner) {
      RegisterEvents();
    }
    public EmptyBoxChartElementGenerator() {
      Parameters.Add(new ValueParameter<IntValue>("X", "The x coordinate of the generated element.", new IntValue(0)));
      Parameters.Add(new ValueParameter<IntValue>("Y", "The y coordinate of the generated element.", new IntValue(0)));
      Parameters.Add(new ValueParameter<IntValue>("RowSpan", "The number of rows occupied by the generated element.", new IntValue(1)));
      Parameters.Add(new ValueParameter<IntValue>("ColSpan", "The number of columns occupied by the generated element.", new IntValue(1)));
      RegisterEvents();
      if (GetType() == typeof(EmptyBoxChartElementGenerator))
        UpdateName();
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EmptyBoxChartElementGenerator(this, cloner);
    }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEvents();
    }
    #endregion

    private void RegisterEvents() {
      XParameter.ToStringChanged += new EventHandler(Parameters_Changed);
      YParameter.ToStringChanged += new EventHandler(Parameters_Changed);
      RowSpanParameter.ToStringChanged +=new EventHandler(Parameters_Changed);
      ColSpanParameter.ToStringChanged += new EventHandler(Parameters_Changed);
    }

    private void Parameters_Changed(object sender, EventArgs e) {
      UpdateName();
    }

    protected virtual string CalculateName() {
      var sb = new StringBuilder();
      sb.AppendFormat("({0}, {1})", X, Y);
      if (RowSpan != 1 || ColSpan != 1)
        sb.AppendFormat("+[{0}, {1}]", ColSpan, RowSpan);
      return sb.ToString();
    }

    protected void UpdateName() {
      name = CalculateName();
      OnNameChanged();
    }

    public virtual void Draw(IRun run, Graphics g) { }
  }
}
